import 'dart:convert' as convert;
import 'dart:io';
import 'dart:typed_data';
import 'dart:ui' as ui;

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:save_image/save_image.dart';
import 'package:zhiying_base_widget/dialog/loading/loading.dart';
import 'package:zhiying_base_widget/pages/goods_share_page/goods_share_content_bloc.dart';
import 'package:zhiying_base_widget/pages/goods_share_page/goods_share_editor/goods_share_editor.dart';
import 'package:zhiying_base_widget/pages/goods_share_page/goods_share_image/goods_share_image_model.dart';
import 'package:zhiying_base_widget/pages/goods_share_page/models/goods_share_poster_model.dart';
import 'package:zhiying_base_widget/pages/goods_share_page/models/goods_share_temp_add_model.dart';
import 'package:zhiying_base_widget/pages/goods_share_page/widgets/good_share_msg.dart';
import 'package:zhiying_base_widget/pages/goods_share_page/widgets/goods_share_edit.dart';
import 'package:zhiying_base_widget/pages/goods_share_page/widgets/goods_share_image.dart';
import 'package:zhiying_base_widget/pages/goods_share_page/widgets/goods_share_item.dart';
import 'package:zhiying_base_widget/template/goods_share_template/goods_share_template.dart';
import 'package:zhiying_base_widget/utils/image_download_util/image_download_util.dart';
import 'package:zhiying_base_widget/widgets/public/custom_button/custom_bottom.dart';
import 'package:zhiying_base_widget/widgets/public/custom_button/custom_button_model.dart';
import 'package:zhiying_base_widget/widgets/share/models/share_data_model.dart';
import 'package:zhiying_base_widget/widgets/share/share_alert.dart';
import 'package:zhiying_comm/util/base_bloc.dart';
import 'package:zhiying_comm/zhiying_comm.dart';

class GoodsShareImage extends StatefulWidget {
  final Map<String, dynamic> model;
  final Map<String, dynamic> params;
  final String shareType;

  const GoodsShareImage(this.model, this.params, this.shareType, {Key key}) : super(key: key);

  @override
  _GoodsShareImageState createState() => _GoodsShareImageState();
}

class _GoodsShareImageState extends State<GoodsShareImage> {
  @override
  Widget build(BuildContext context) {
    return BlocProvider<GoodsShareContentBloc>(
      bloc: GoodsShareContentBloc(),
      child: _GoodsShareImageContent(widget.model, widget.params, widget.shareType),
    );
  }
}

class _GoodsShareImageContent extends StatefulWidget {
  final Map<String, dynamic> model;
  final Map<String, dynamic> params;
  final String shareType;

  const _GoodsShareImageContent(this.model, this.params, this.shareType, {Key key}) : super(key: key);

  @override
  _GoodsShareImageContentState createState() => _GoodsShareImageContentState();
}

class _GoodsShareImageContentState extends State<_GoodsShareImageContent> with AutomaticKeepAliveClientMixin {
  GlobalKey _globalKey = GlobalKey();
  GoodsShareImageModel _style;
  Map<int, String> _images = Map();
  List<Widget> widgets = List();
  String _content = '';
  bool _isContentSelected = false;
  bool _isPosterSelected = false;

  GoodsShareContentBloc _bloc;

  // 分享数据
  final ShareDataModel _shareModel = ShareDataModel();

  @override
  // 切换Tab导航栏保持页面状态
  bool get wantKeepAlive => true;

  @override
  void initState() {
    String data = widget.model['data'];
    var d = convert.jsonDecode(data);
    _style = GoodsShareImageModel.fromJson(Map<String, dynamic>.from(d));
    // _content = widget.datas.shareContent;

    _bloc = BlocProvider.of<GoodsShareContentBloc>(context);
    _bloc.loadData(widget.shareType, widget.params);

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<GoodsShareTempDataModel>(
        stream: _bloc.outData,
        builder: (BuildContext context, AsyncSnapshot snapshot) {
          GoodsShareTempDataModel data = snapshot.data;
          widgets.clear();
          if (data != null) {
            if (_content == null || _content == '') {
              _content = data.shareContent;
            }
            widgets.add(GoodsShareItem(
              imgHeader: _style.cssList.headerImage,
              imgCheck: _style.cssList.unChooseImage,
              imgCheckSelected: _style.cssList.chooseImage,
              child: GoodsShareEdit(
                _content,
                data,
                imgCheck: _style.cssList.unChooseImage,
                imgCheckSelected: _style.cssList.chooseImage,
                onCopyClick: () {
                  Clipboard.setData(ClipboardData(text: _content));
                  Fluttertoast.showToast(msg: '文案已复制');
                },
                onEditorClick: () {
                  Fluttertoast.showToast(msg: '正在开发中');
                  return;
                  Navigator.push(context, CupertinoPageRoute(builder: (context) => GoodsShareEditor(_style)));
                },
                onTempChange: (temp) {
                  setState(() {
                    _content = data.shareContent + temp;
                  });
                },
              ),
              onSelected: (isSelected) {
                _isContentSelected = isSelected;
                // _shareModel.content = isSelected ? _content : '';
              },
            ));
            for (String message in _style.customContent) {
              widgets.add(
                GoodsShareItem(
                  imgHeader: _style.cssList.headerImage,
                  imgCheck: _style.cssList.unChooseImage,
                  imgCheckSelected: _style.cssList.chooseImage,
                  enabled: false,
                  child: GoodsShareMessage(message),
                ),
              );
            }
            widgets.add(
              GoodsShareItem(
                imgHeader: _style.cssList.headerImage,
                imgCheck: _style.cssList.unChooseImage,
                imgCheckSelected: _style.cssList.chooseImage,
                enabled: true,
                child: Container(
                  // color: Colors.redAccent,
                  width: 100,
                  height: 140,
                  padding: EdgeInsets.only(left: 25),
                  child: Transform.scale(
                    alignment: Alignment.centerLeft,
                    scale: 0.2,
                    child: StreamBuilder<GoodsSharePosterModel>(
                      stream: _bloc.outPoster,
                      builder: (BuildContext context, AsyncSnapshot snapshot) {
                        GoodsSharePosterModel poster = snapshot.data;
                        return poster == null
                            ? Container()
                            : GoodsShareTemplate(
                                poster,
                                contentKey: _globalKey,
                              );
                      },
                    ),
                  ),
                ),
                onSelected: (isSelected) {
                  _isPosterSelected = isSelected;
                },
              ),
            );

            List<String> moreImageList = List();
            try {
              moreImageList = List.from(widget?.params['image_url_list']).map((e) => e.toString()).toList();
            } catch (e, s) {
              print(e);
              print(s);
            }

            _style.customImage.addAll(moreImageList);
            _style.customImage = _style.customImage.reversed.toList();
            for (int index = 0; index < _style.customImage.length; index++) {
              String image = _style.customImage[index];
              // 分享图片
              widgets.add(
                GoodsShareItem(
                  imgHeader: _style.cssList.headerImage,
                  imgCheck: _style.cssList.unChooseImage,
                  imgCheckSelected: _style.cssList.chooseImage,
                  enabled: true,
                  child: GoodsShareImageWidget(image),
                  onSelected: (isSelected) {
                    if (_images.containsKey(index)) {
                      _images.remove(index);
                    } else {
                      _images[index] = image;
                    }
                  },
                ),
              );
            }
          }

          return Column(
            children: <Widget>[
              Expanded(
                child: Container(
                  margin: EdgeInsets.only(bottom: 4),
                  padding: EdgeInsets.only(top: 10),
                  decoration: BoxDecoration(color: Colors.white, borderRadius: BorderRadius.only(bottomLeft: Radius.circular(8), bottomRight: Radius.circular(8))),
                  child: SingleChildScrollView(
                    child: Column(crossAxisAlignment: CrossAxisAlignment.start, mainAxisAlignment: MainAxisAlignment.start, children: widgets),
                  ),
                ),
              ),
              _createBottom(),
            ],
          );
        });
  }

  Widget _createBottom() {
    Map<String, dynamic> shareImages = _style.cssList.shareImage;
    if (shareImages == null) {
      return Container();
    }
    CustomButtonModel btnCopy = CustomButtonModel.fromJson(Map<String, dynamic>.from(shareImages['copy_btn']));
    CustomButtonModel btnSave = CustomButtonModel.fromJson(Map<String, dynamic>.from(shareImages['save_btn']));
    CustomButtonModel btnShare = CustomButtonModel.fromJson(Map<String, dynamic>.from(shareImages['share_btn']));
    return SafeArea(
      child: Container(
        margin: EdgeInsets.only(left: 12.5, right: 12.5, top: 4, bottom: 4),
        child: Row(
          children: <Widget>[
            btnCopy == null
                ? Container()
                : GestureDetector(
                    child: Container(
                        width: 95,
                        height: 36,
                        decoration: BoxDecoration(
                          borderRadius: BorderRadius.circular(18),
                        ),
                        child: CustomButton(btnCopy)),
                    onTap: () {
                      Clipboard.setData(ClipboardData(text: _content));
                      Fluttertoast.showToast(msg: '文案已复制');
                    },
                  ),
            btnSave == null
                ? Container()
                : GestureDetector(
                    child: Container(
                      margin: EdgeInsets.only(left: 5),
                      width: 95,
                      height: 36,
                      decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(18),
                      ),
                      child: CustomButton(btnSave),
                    ),
                    onTap: () {
                      _saveImages();
                    },
                  ),
            btnShare == null
                ? Container()
                : Expanded(
                    child: GestureDetector(
                      child: Container(
                          margin: EdgeInsets.only(left: 5),
                          // width: 95,
                          height: 36,
                          decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(18),
                          ),
                          child: CustomButton(btnShare)),
                      onTap: () {
                        _showShareAlert();
                      },
                    ),
                  ),
          ],
        ),
      ),
    );
  }

  void _saveImages() async {
    Loading.show(context, msg: '正在保存');

    await _updateModel();

    List<String> images = List();
    for (int index = 0; index < _style.customImage.length; index++) {
      if (_images.containsKey(index)) {
        images.add(_images[index]);
      }
    }
    if (images.length > 0) {
      _shareModel.image = images;
    } else {
      _shareModel.image = null;
    }

    if ((_shareModel.image == null || _shareModel.image.length == 0) && _shareModel.poster == null) {
      Fluttertoast.showToast(msg: '请选择要保存的图片');
      Loading.dismiss();
      return;
    }

    //检查是否有存储权限
    var status = await Permission.photos.status;
    if (status != PermissionStatus.granted) {
      status = await Permission.photos.request();
    }
    if (status == PermissionStatus.denied) {
      Fluttertoast.showToast(msg: '暂无权限，保存失败');
      Loading.dismiss();
      return null;
    }

    // 检查并请求权限
    status = await Permission.storage.status;
    if (status != PermissionStatus.granted) {
      status = await Permission.storage.request();
    }
    if (status == PermissionStatus.denied) {
      Fluttertoast.showToast(msg: '暂无权限，分享失败');
      Loading.dismiss();
      return null;
    }

    bool isSaveSuccess = false;
    if (_shareModel.poster != null) {
      isSaveSuccess = await SaveImage.save(imageBytes: _shareModel.poster);
    }

    List<String> paths = await ImageDownloadUtil.download(images);
    for (var item in paths) {
      Uint8List data = File(item).readAsBytesSync();
      isSaveSuccess = await SaveImage.save(imageBytes: data);
    }
    if (isSaveSuccess) {
      Fluttertoast.showToast(msg: '保存成功');
    } else {
      Fluttertoast.showToast(msg: '保存失败');
    }

    Loading.dismiss();
  }

  Future<ShareDataModel> _updateModel() async {
    if (_isPosterSelected) {
      BuildContext buildContext = _globalKey.currentContext;
      if (null != buildContext) {
        RenderRepaintBoundary boundary = buildContext.findRenderObject();
        ui.Image image = await boundary.toImage(pixelRatio: 4);
        // 注意：png是压缩后格式，如果需要图片的原始像素数据，请使用rawRgba
        Loading.show(context);
        ByteData byteData = await image.toByteData(format: ui.ImageByteFormat.png);
        Uint8List pngBytes = byteData.buffer.asUint8List();
        File file = await EncodeUtil.compressImage(context,images: pngBytes, size: 200);
        Loading.dismiss();
        _shareModel.poster = file.readAsBytesSync();
      }
    } else {
      _shareModel.poster = null;
    }
    _shareModel.content = _isContentSelected ? _content : '';
    return _shareModel;
  }

  // 弹出分享框
  void _showShareAlert() async {
    await _updateModel();
    List<String> images = List();
    for (int index = 0; index < _style.customImage.length; index++) {
      if (_images.containsKey(index)) {
        images.add(_images[index]);
      }
    }
    if (images.length > 0) {
      _shareModel.image = images;
    } else {
      _shareModel.image = null;
    }

    if ((_shareModel.title == null || _shareModel.title == '') &&
        (_shareModel.image == null || _shareModel.image.length == 0) &&
        (_shareModel.content == null || _shareModel.content == '') &&
        (_shareModel.url == null || _shareModel.url == '') &&
        (_shareModel.poster == null)) {
      Fluttertoast.showToast(msg: '请选择要分享的内容');
      return;
    }

    // ============= 复制文案 =================
    Clipboard.setData(ClipboardData(text: _content));

    showCupertinoModalPopup(
      context: context,
      builder: (context) => ShareAlert(
        _shareModel,
        'pub.flutter.share_icon',
        isContentShow: true,
        // child: GoodsShareAlertContent(),
      ),
    );
  }
}
