import 'dart:convert' as convert;
import 'dart:ui' as ui;

import 'package:event_bus/event_bus.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:zhiying_base_widget/widgets/custom/goods/category_goods/category_goods_callback.dart';
import 'package:zhiying_base_widget/widgets/home/home_goods/bloc/home_goods_header_bloc.dart';
import 'package:zhiying_base_widget/widgets/home/home_goods/home_goods_creater.dart';
import 'package:zhiying_base_widget/widgets/home/home_goods/models/home_goods_header_model.dart';
import 'package:zhiying_base_widget/widgets/home/home_goods/models/home_goods_style_model.dart';
import 'package:zhiying_comm/util/base_bloc.dart';
import 'package:zhiying_comm/zhiying_comm.dart';

import 'custom_goods.dart';

class CustomGoodsHeaderDelegate extends SliverPersistentHeaderDelegate {
  final Widget child;
  CustomGoodsHeaderDelegate(this.child) : super();

  @override
  Widget build(BuildContext context, double shrinkOffset, bool overlapsContent) {
    print('${shrinkOffset.toString()}  ${overlapsContent.toString()}');
    return child;
  }

  @override
  double get maxExtent => 64.0;

  @override
  double get minExtent => 64.0;

  @override
  bool shouldRebuild(SliverPersistentHeaderDelegate oldDelegate) => false; // 如果内容需要更新，设置为true
}

class CustomGoodsHeader extends StatelessWidget {
  final Map<String, dynamic> model;
  final CategoryGoodsOnClickCallBack callBack;


  const CustomGoodsHeader(
    this.model,
      this.callBack, {
    Key key,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return BlocProvider<HomeGoodsHeaderBloc>(
      bloc: HomeGoodsHeaderBloc(),
      child: CustomGoodsHeaderContainer(model, callBack),
    );
  }
}

class CustomGoodsHeaderContainer extends StatefulWidget {
  final Map<String, dynamic> model;
  final CategoryGoodsOnClickCallBack callBack;

  CustomGoodsHeaderContainer(this.model, this.callBack, {Key key}) : super(key: key);

  @override
  _CustomGoodsHeaderContainerState createState() => _CustomGoodsHeaderContainerState();
}

class _CustomGoodsHeaderContainerState extends State<CustomGoodsHeaderContainer> with TickerProviderStateMixin {
  HomeGoodsStyleModel _style;
  List<Widget> _widgets = List();

  TabController _tabController;
  int _currentIndex = 0;

  @override
  void initState() {
    String d = widget.model['data'];
    dynamic json = convert.jsonDecode(d);
    _style = HomeGoodsStyleModel.fromJson(Map<String, dynamic>.from(json));

    _tabController = TabController(length: _style?.recommendList?.length ?? 0, vsync: this);
    if (_style.recommendList.first != null) {
      // TODO
      widget.callBack.onTap(0, _style.recommendList.first.data);
    }
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    _widgets.clear();

    for (int index = 0; index < _style.recommendList.length; index++) {
      _widgets.add(_HomeGoodsHeaderItem(
        _style.recommendList[index],
        titleTextColor: index == _currentIndex ? (HexColor.fromHex(_style.tilteTextSelectedColor ?? '#FF4242')) : (HexColor.fromHex(_style.tilteTextColor ?? '#333333')),
        subtitleTextColor: index == _currentIndex ? (HexColor.fromHex(_style.subtitleTextSelectedColor ?? '#ffffff')) : (HexColor.fromHex(_style.subtitleTextColor ?? '#999999')),
        subtitleBgColor: index == _currentIndex ? HexColor.fromHex(_style.subtitleBgColor ?? '#FF4242') : Colors.transparent,
      ));
    }

    return Container(
      width: double.infinity,
      height: double.infinity,
      color: Color(0xfff9f9f9),
      // color: Colors.yellow,
      child: TabBar(
        indicatorColor: Colors.transparent,
        controller: _tabController,
        isScrollable: true,
        tabs: _widgets,
        onTap: (index) {
          //TODO
          widget.callBack.onTap(index, _style.recommendList[index].data);
          setState(() {
            _currentIndex = index;
          });
        },
      ),
    );
  }
}

class _HomeGoodsHeaderItem extends StatelessWidget {
  final HomeGoodsHeaderModel model;
  final ui.Color titleTextColor;
  final ui.Color subtitleTextColor;
  final ui.Color subtitleBgColor;

  const _HomeGoodsHeaderItem(
    this.model, {
    Key key,
    this.titleTextColor,
    this.subtitleBgColor,
    this.subtitleTextColor,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
        // width: 60,
        // height: 54,
        child: Center(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Text(
            model.title,
            style: TextStyle(color: titleTextColor ?? ui.Color(0xff333333), fontSize: 17, fontWeight: FontWeight.bold),
          ),
          model.subtitle == null || model.subtitle == ''
              ? Container()
              : Container(
                  margin: EdgeInsets.only(top: 4),
                  padding: EdgeInsets.only(left: 6, right: 6, top: 2, bottom: 2),
                  decoration: BoxDecoration(color: subtitleBgColor ?? Colors.transparent, borderRadius: BorderRadius.circular(18)),
                  child: Text(
                    model.subtitle ?? '',
                    style: TextStyle(fontSize: 10, color: subtitleTextColor ?? Color(0xff999999)),
                    textAlign: TextAlign.center,
                  ),
                ),
        ],
      ),
    ));
  }
}
