import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:zhiying_base_widget/widgets/goods_details/detail_img/bloc/goods_details_img_bloc.dart';
import 'package:zhiying_base_widget/widgets/goods_details/detail_img/goods_details_img_sk.dart';
import 'package:zhiying_base_widget/widgets/goods_details/detail_img/model/goods_details_img_model.dart';
import 'package:zhiying_comm/zhiying_comm.dart';
import 'package:shimmer/shimmer.dart';
import 'package:cached_network_image/cached_network_image.dart';
import 'package:zhiying_base_widget/widgets/goods_details/detail_img/bloc/goods_details_img_repository.dart';

///
/// 商品详情图片
///
class GoodsDetailsImgWidget extends StatelessWidget {
  final Map<String, dynamic> model;

  const GoodsDetailsImgWidget(this.model);

  @override
  Widget build(BuildContext context) {
    return BlocProvider<GoodsDetailsImgBloc>(
      create: (_) => GoodsDetailsImgBloc(repository: GoodsDetailsImgRepository()),
      //..add(GoodsDetailsImgInitEvent(model: model)),
      child: GoodsDetailsImgWidgetContainer(
        model,
        key: UniqueKey(),
      ),
    );
  }
}

class GoodsDetailsImgWidgetContainer extends StatefulWidget {
  final Map<String, dynamic> model;

  const GoodsDetailsImgWidgetContainer(this.model, {Key key}) : super(key: key);

  @override
  _GoodsDetailsImgWidgetContainerState createState() => _GoodsDetailsImgWidgetContainerState();
}

class _GoodsDetailsImgWidgetContainerState extends State<GoodsDetailsImgWidgetContainer> {
  @override
  void initState() {
    BlocProvider.of<GoodsDetailsImgBloc>(context).add(GoodsDetailsImgInitEvent(model: widget?.model));
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return BlocConsumer<GoodsDetailsImgBloc, GoodsDetailsImgState>(
      listener: (BuildContext context, GoodsDetailsImgState state) {
        if (state is GoodsDetailsImgErrorState) {
          print('数据加载出错');
        }
      },
      buildWhen: (previous, current) {
        /// 数据加载出错不进行build
        if (current is GoodsDetailsImgErrorState) {
          return false;
        }
        return true;
      },
      builder: (context, state) {
        print('currente state = $state');
        if (state is GoodsDetailsImgLoadedState) {
          if (!EmptyUtil.isEmpty(state.model)) {
            return _getMainWidget(state.model);
          }
        }
        // 骨架屏
        return GoodsDetailsImgSkeleton();
      },
    );
  }

  ///
  Widget _getMainWidget(GoodsDetailsImgModel model) {
    return Column(
      children: <Widget>[
        /// 标题
        _getTitleWidget(model),

        /// 图片列表
        _getImgListWidget(model),
      ],
    );
  }

  /// 图片列表
  Widget _getImgListWidget(GoodsDetailsImgModel model) {
    int length = model?.image_detail_list?.length ?? 0;
    if (length > 0) {
      return Column(
        children: model.image_detail_list.map((item) {
          return GestureDetector(
              onTap: () {
                if (item != null) {
                  PhotoPreview.showPhotoPreviewByimages(context, [item], currentIndex: 0, heroTagSuffix: "bottom");
                }
              },
              child: Hero(
                tag: item + "bottom",
                child: CachedNetworkImage(
                  imageUrl: item ?? '',
                  fit: BoxFit.fitWidth,
                  // 默认default图片
                  placeholder: (context, url) {
                    return AspectRatio(
                      aspectRatio: 1 / 1,
                      child: shimmerWidget(
                        height: double.infinity,
                        width: double.infinity,
                      ),
                    );
                  },
                  errorWidget: (BuildContext context, String url, dynamic error) {
                    return Container();
                  },
                ),
              ));
        }).toList(),
      );
    } else {
      return Container();
    }
  }

  /// 标题Widget
  Widget _getTitleWidget(GoodsDetailsImgModel model) {
    return Container(
      decoration: BoxDecoration(
        color: HexColor.fromHex(model?.bgColor),
        borderRadius: BorderRadius.only(
          topRight: Radius.circular(ParseUtil.stringParseDouble(model?.topRightRadius)),
          topLeft: Radius.circular(ParseUtil.stringParseDouble(model?.topLeftRadius)),
          bottomRight: Radius.circular(ParseUtil.stringParseDouble(model?.bottomRightRadius)),
          bottomLeft: Radius.circular(ParseUtil.stringParseDouble(model?.bottomLeftRadius)),
        )
      ),
      margin: EdgeInsets.only(top: ParseUtil.stringParseDouble(model?.topMargin), left: ParseUtil.stringParseDouble(model?.leftRightMargin), right: ParseUtil.stringParseDouble(model?.leftRightMargin)),
      padding: const EdgeInsets.only(top: 15, bottom: 15, left: 12.5, right: 12.5),
      width: double.infinity,
      child: _getLeftWidget(model),
    );
  }

  /// 左边图标加文字
  Widget _getLeftWidget(GoodsDetailsImgModel model) {
    return Row(
      children: <Widget>[
        CachedNetworkImage(
          imageUrl: model?.icon ?? '',
          width: 12,
        ),
        const SizedBox(width: 7),
        Text(model?.title ?? '商品详情',
            style: TextStyle(color: HexColor.fromHex(model?.titleColor ?? '#333333'), fontSize: 12)),
      ],
    );
  }


}

/// 骨架视图
Widget shimmerWidget({double width, double height, double radius = 0}) {
  //修改后返回默认图片
  return Image.asset(
    'assets/images/occupation_map/occupation_map.png',
    package: 'zhiying_base_widget',
    width: width,
    height: height,
    fit: BoxFit.fill,
  );
}